@@ -14,7 +14,7 @@ import itertools
 import math
 import operator
 import uuid
-from typing import Any, Mapping, MutableMapping, Sequence, TypeVar
+from typing import Any, Iterable, Mapping, MutableMapping, Sequence, TypeVar
 
 import ibis
 import ibis.expr.datatypes as dt
@@ -23,6 +23,7 @@ import ibis.expr.schema as sch
 import ibis.expr.types as ir
 import toolz
 from ibis import util
+from packaging import version
 
 from ibis_substrait.proto.substrait.ibis import algebra_pb2 as stalg
 from ibis_substrait.proto.substrait.ibis import type_pb2 as stt
@@ -30,15 +31,11 @@ from ibis_substrait.proto.substrait.ibis import type_pb2 as stt
 from .core import SubstraitCompiler, _get_fields
 
 IBIS_4 = False
-try:
-    from ibis.util import to_op_dag
+if version.parse(ibis.__version__) >= version.parse("4.0.0"):
+    IBIS_4 = True
 
-    # Bit of a hack -- there is no ops.CountStar in Ibis 3.x but to register it
-    # for 4.x below, it can't be undefined here.
-    # We remap it to ops.Count just to avoid an attribute error, it will never
-    # be used to route in Ibis 3.x because it doesn't exist.
-    ops.CountStar = ops.Count
-except ImportError:
+
+if IBIS_4:
     import warnings
 
     from ibis.common.graph import toposort
@@ -48,8 +45,18 @@ except ImportError:
         message="`Node.op` is deprecated",
         category=FutureWarning,
     )
+else:
+    from ibis.util import to_op_dag
 
-    IBIS_4 = True
+    # There is no ops.CountStar in Ibis 3.x but to register it for 4.x below, it
+    # can't be undefined here.
+    # We remap it to ops.Count just to avoid an attribute error, it will never
+    # be used to route in Ibis 3.x because it doesn't exist.
+    ops.CountStar = ops.Count
+    # ops.ValueOp renamed to ops.Value in Ibix 3.2
+    # We manually add ops.Value here for Ibis 3.0 compatibility
+    if hasattr(ops, "ValueOp"):
+        ops.Value = ops.ValueOp
 
 T = TypeVar("T")
 
@@ -775,50 +782,37 @@ def selection(
             )
         )
 
-    # TODO: there has to be a better way to get a list of source tables
-    # underlying an expression
-    # TODO: settle on a better source table definition than "PhysicalTable with
-    # a schema"
-    if IBIS_4:
-        # projection / emit
-        selections = [
-            col
-            for sel in (x.to_expr() for x in op.selections)  # map ops to exprs
-            for col in (
-                sel.get_columns(sel.columns) if isinstance(sel, ir.TableExpr) else [sel]
+    if selections := _get_selections(op):
+
+        rels = [
+            (attr, getattr(relation, attr))
+            for attr in (
+                "aggregate",
+                "cross",
+                "extension_leaf",
+                "extension_multi",
+                "extension_single",
+                "fetch",
+                "filter",
+                "join",
+                "project",
+                "read",
+                "set",
+                "sort",
             )
         ]
-        source_tables = {
-            t
-            for t in toposort(op).keys()
-            if isinstance(t, ops.PhysicalTable) and hasattr(t, "schema")
-        }
-    else:
-
-        # projection / emit
-        selections = [
-            col
-            for sel in op.selections
-            for col in (
-                sel.get_columns(sel.columns) if isinstance(sel, ir.TableExpr) else [sel]
-            )
-        ]
-        source_tables = {
-            t
-            for t in to_op_dag(op.to_expr()).keys()
-            if isinstance(t, ops.PhysicalTable) and hasattr(t, "schema")
-        }
-
-    mapping_counter = itertools.count(sum(len(t.schema) for t in source_tables))
-    if selections:
-
-        if relation.project.common.ListFields():
-            # if there is already an `emit` in RelCommon then we're stacking
-            # projections and we need to update the output_mapping to refer to
-            # the number of fields present in the most recent emit
-            mapping_counter = itertools.count(
-                len(relation.project.common.emit.output_mapping)
-            )
+        for relname, rel in rels:
+            if relname == "aggregate" and rel.measures:
+                mapping_counter = itertools.count(
+                    len(rel.measures) + len(rel.groupings)
+                )
+                break
+            elif output_mapping := rel.common.emit.output_mapping:
+                mapping_counter = itertools.count(len(output_mapping))
+                break
+        else:
+            source_tables = _find_parent_tables(op)
+            mapping_counter = itertools.count(sum(len(t.schema) for t in source_tables))
 
         relation = stalg.Rel(
             project=stalg.ProjectRel(
@@ -860,6 +854,48 @@ def selection(
     return relation
 
 
+def _get_selections(op: ops.Selection) -> Sequence[ir.Column]:
+    if IBIS_4:
+        # projection / emit
+        selections = [
+            col
+            for sel in (x.to_expr() for x in op.selections)  # map ops to exprs
+            for col in (
+                sel.get_columns(sel.columns) if isinstance(sel, ir.TableExpr) else [sel]
+            )
+        ]
+    else:
+        # projection / emit
+        selections = [
+            col
+            for sel in op.selections
+            for col in (
+                sel.get_columns(sel.columns) if isinstance(sel, ir.TableExpr) else [sel]
+            )
+        ]
+
+    return selections
+
+
+def _find_parent_tables(op: ops.Selection) -> set[ir.Table]:
+    # TODO: settle on a better source table definition than "PhysicalTable with
+    # a schema"
+    if IBIS_4:
+        source_tables = {
+            t
+            for t in toposort(op).keys()
+            if isinstance(t, ops.PhysicalTable) and hasattr(t, "schema")
+        }
+    else:
+        source_tables = {
+            t
+            for t in to_op_dag(op.to_expr()).keys()
+            if isinstance(t, ops.PhysicalTable) and hasattr(t, "schema")
+        }
+
+    return source_tables
+
+
 @functools.singledispatch
 def _translate_join_type(op: ops.Join) -> stalg.JoinRel.JoinType.V:
     raise NotImplementedError(f"join type `{type(op).__name__}` not implemented")
@@ -1086,11 +1122,16 @@ def _contains(
     compiler: SubstraitCompiler | None = None,
     **kwargs: Any,
 ) -> stalg.Expression:
+    options = (
+        op.options
+        if isinstance(op.options, Iterable)
+        else ibis.util.promote_list(op.options)
+    )
     return stalg.Expression(
         singular_or_list=stalg.Expression.SingularOrList(
             value=translate(op.value, compiler=compiler, **kwargs),
             options=[
-                translate(value, compiler=compiler, **kwargs) for value in op.options
+                translate(value, compiler=compiler, **kwargs) for value in options
             ],
         )
     )
@@ -1210,3 +1251,25 @@ def _clip(
             **kwargs,
         )
     raise AssertionError()
+
+
+@translate.register(ops.TableArrayView)
+def _table_array_view(
+    op: ops.TableArrayView,
+    expr: ir.TableExpr | None = None,
+    *,
+    compiler: SubstraitCompiler | None = None,
+    **kwargs: Any,
+) -> stalg.Expression:
+    return translate(op.table, compiler=compiler, **kwargs)
+
+
+@translate.register(ops.SelfReference)
+def _self_reference(
+    op: ops.SelfReference,
+    expr: ir.TableExpr | None = None,
+    *,
+    compiler: SubstraitCompiler | None = None,
+    **kwargs: Any,
+) -> stalg.Expression:
+    return translate(op.table, compiler=compiler, **kwargs)
